
// >>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

enum side {
	K_SIDE_TOP=1,
	K_SIDE_BOTTOM=2,
	K_SIDE_LEFT=4,
	K_SIDE_RIGHT=8
};

enum anchor {
	K_ANCHOR_NORTH=1,
	K_ANCHOR_SOUTH=2,
	K_ANCHOR_WEST=8,
	K_ANCHOR_EAST=16
};

enum fill {
	K_FILL_X=1,
	K_FILL_Y=2,
	K_FILL_BOTH=4
};

enum pack {
	K_PACK_DEFAULT,
	K_PACK_AFTER,
	K_PACK_BEFORE,
	K_PACK_IN
};

class KOwner;

class KParcel {
public:

	KParcel();	
	virtual ~KParcel();
	
	virtual BRect Bounds()=0;
	
	// reshape the view so it is contained by frame
	virtual BRect Reshape(BView *root,BRect frame)=0;
	
	// try to extend the container by x and y
	virtual void ExtendBy(float x,float y,KOwner *source)=0;
	
	virtual void GetPreferredParcelSize(float *width,float *height)=0;
	
	//virtual const BRect ContainerFrame()=0;
	
	virtual int32 Extendable()=0;
	
	inline KOwner *Owner() {return fOwner;}

private:
	friend KOwner;
	KOwner *fOwner;

};

class KOwner {
public:

	KOwner(KParcel *super_parcel,KParcel *parcel=NULL);
	virtual ~KOwner();
	
	inline KParcel *SuperParcel() {return fSuperParcel;};
	
	inline KParcel *Parcel() {return fParcel;}
	void SetParcel(KParcel *parcel);
	
	inline KOwner *NextSibling() {return fNextSibling;}

private:
	KParcel	*fParcel,*fSuperParcel;
	KOwner *fNextSibling;
};

class KWarper : public KParcel
{
	KWarper(BView *view,anchor anchor,fill fill,bool expand=false);
	
	inline const BView *View() {return fView;}
	inline const anchor Anchor() {return fAnchor;}
	inline const bool Expand() {return fExpand;} 
	
	virtual BRect Bounds();
	
	// reshape the view so it is contained by frame
	virtual BRect Reshape(BView *root,BRect frame);
	
	// try to extend the container by x and y
	virtual void ExtendBy(float x,float y,KOwner *source);
	
	virtual void GetPreferredContainsSize(float *width,float *height);
	
	//virtual const BRect ContainerFrame();
private:

	BView *fView;

	anchor fAnchor;
	fill fFill;
	bool fExpand;
};

class KWarperNode : public KParcel
{
public:

	KWarperNode();
	KWarperNode(KParcel *master,side side);
	
	inline const KOwner *Master() {return &fMaster;}
	inline const KParcel *MasterParcel() {return fMaster.Parcel();}
	inline const KOwner *Slave() {return &fSlave;}
	inline const KParcel *SlaveParcel() {return fSlave.Parcel();}
	
	inline const side Side() {return fSide;}
	inline const orientation Posture() {return (fSide&(K_SIDE_TOP|K_SIDE_BOTTOM)?B_HORIZONTAL:B_VERTICAL);}
	
	virtual BRect Bounds();

	// reshape the view so it is contained by frame
	virtual BRect Reshape(BView *root,BRect frame);
	
private:

	KOwner fMaster,fSlave;
	side fSide;
	
};

class KPackerView : public BView, public KOwner
{

	KPacker(BRect frame,
		  const char *name,
		  uint32 resizeMask=B_FOLLOW_ALL,
		  uint32 flags);
		  
	virtual ~KPackerView();

	const KOwner *LastOwner();
	
	KContainer *PackChild(BView *view,side side,anchor anchor,fill fill,bool expand);
	
	virtual void FrameResized(float width,float height);
	virtual void AttachedToWindow();
	
private:
	
	LastOwner *fLastOwner;
		
};
