//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998,1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>
#include <math.h>

#include <InterfaceKit.h>
#include <SupportDefs.h>
#include <ClassInfo.h>

#include "tab_view.h"
#include "interface_kit_tools.h"
#include "pack_view.h"

KTab::KTab(BView * tabView):BTab(tabView){fFirstTime=true;};
KTab::KTab(BMessage *archive):BTab(archive){fFirstTime=true;}; 
    
void KTab::Deselect()
{
	BTab::Deselect();
}
    
void KTab::Select(BView *owner)
{
	if (!fFirstTime) BTab::Select(owner);
	// resize the view so it fills the owner correctly
	if (View()!=NULL&&owner->Window()!=NULL) {
    	BRect rect=owner->Bounds();
    	rect.InsetBy(5,5);// looks nice ?
    	View()->MoveTo(rect.left,rect.top);
    	View()->ResizeTo(rect.Width(),rect.Height());
    }
	if (fFirstTime) {
		BTab::Select(owner);// this will cause the view to be attached...
		fFirstTime=false;
	}
}

KTabView::KTabView(BRect frame, const char *name, 
         button_width width, 
         uint32 resizingMode, 
         uint32 flags):
     	BTabView(frame,name,width,resizingMode,flags)
{
	fLeftTabsOffset=10;
}

void KTabView::AllAttached()
{}
void KTabView::AttachedToWindow()
{
	SetTabWidth(B_WIDTH_FROM_WIDEST);
	BTabView::AttachedToWindow();
}

void KTabView::AddTab(BView *target, BTab *tab)
{
	if (tab!=NULL)
		BTabView::AddTab(target,tab);
	else {
		//BTabView::AddTab(target,new KTab(target));
	
		KPackView *pack=dynamic_cast<KPackView *>(target);
		if (pack==NULL) {
			pack=new KPackView(target->Bounds(),target->Name());
			pack->PackChild(target,K_FULL,K_LEFT,K_FILL_BOTH);
		}
		BTabView::AddTab(pack,new KTab(pack));
	
	}
}

void KTabView::GetPreferredSize(float *width,float *height)
{
	printf("%s::GetPreferredSize\n",Name());
	//BTabView::GetPreferredSize(width,height);
	BRect frame=TabFrame(CountTabs()-1);
	BView *view=ViewForTab(Selection());
	if (view!=NULL) {
		view->GetPreferredSize(width,height);
		*width=max_c(*width+10,frame.right);
		*height=*height+frame.Height()+30;
	} else {
		BTabView::GetPreferredSize(width,height);
		*width=frame.right;
		*height=300;
	}
}

// DRAWING PART <<<<<<<<<<<<<<<<<<<<<<<<<<<<<

void KTab::DrawTab(BView *owner,BRect frame,tab_position position,bool full)
{
	rgb_color	hi;
	rgb_color	lo;

	// Save the original colors
	
	hi = owner->HighColor();
	lo = owner->LowColor();

	// Draw the label by calling DrawLabel()

	owner->SetHighColor(background_color);
	owner->SetLowColor(text_color);
	DrawLabel(owner, frame);
	
	// 
	
	owner->BeginLineArray(10);
	if (position==B_TAB_FRONT) {
		owner->AddLine(frame.LeftBottom(),BPoint(frame.left+10,frame.top),light_border_color);
		//owner->AddLine(frame.LeftTop(),frame.RightTop(),light_border_color);
		owner->AddLine(BPoint(frame.right-10,frame.top),frame.RightBottom(),dark_border_color);
	} else if (position==B_TAB_FIRST) {
		owner->AddLine(frame.LeftBottom(),BPoint(frame.left+10,frame.top),light_border_color);
	} else {
		//owner->AddLine(frame.LeftTop(),frame.RightTop(),light_border_color);
	}
		if (full) owner->AddLine(BPoint(frame.right-10,frame.top),frame.RightBottom(),dark_border_color);
	
	owner->EndLineArray();
	owner->SetHighColor(hi); 
	owner->SetLowColor(lo); 
}

BRect KTabView::TabFrame(int32 tabIndex) const
{
	BRect frame=BTabView::TabFrame(tabIndex);
	frame.left+=fLeftTabsOffset+tabIndex*10;
	frame.right+=fLeftTabsOffset+10+tabIndex*10;
	return frame;
}

void KTabView::DrawBox(BRect selTabRect)
{
	BeginLineArray(5);
	// right
	AddLine(BPoint(Bounds().right,selTabRect.bottom),Bounds().RightBottom(),dark_border_color);
	// bottom
	AddLine(Bounds().RightBottom(),Bounds().LeftBottom(),dark_border_color);
	// left
	//AddLine(Bounds().LeftBottom(),Bounds().LeftTop(),light_border_color);
	AddLine(Bounds().LeftBottom(),BPoint(Bounds().left,selTabRect.bottom),light_border_color);
	// top 1&2
	AddLine(BPoint(0,selTabRect.bottom),selTabRect.LeftBottom(),light_border_color);
	AddLine(selTabRect.RightBottom(),BPoint(Bounds().right,selTabRect.bottom),light_border_color);
	EndLineArray();
}

BRect KTabView::DrawTabs()
{
	int32 count=CountTabs();
	if (count>0) {
		BRect first=TabFrame(0);
		BRect last=TabFrame(CountTabs()-1);
		SetLowColor(light_border_color);
		StrokeLine(BPoint(first.left+10,0),BPoint(last.right-10,0),B_SOLID_LOW);
	}
	BRect frame=BTabView::DrawTabs();
	return frame;
}
  	