//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998-1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include "tips.h"

KTipsWindow::KTipsWindow(BView *owner,BPoint where,const char *text):
BWindow(BRect(where.x-20,where.y+20,where.x+50,where.y+40),"",B_BORDERED_WINDOW_LOOK,B_FLOATING_SUBSET_WINDOW_FEEL,B_AVOID_FRONT)
{
	BRect bounds=Bounds();
	bounds.InsetBy(3,3);
	BTextView *bg=new BTextView(Bounds(),"",bounds,B_FOLLOW_ALL,B_WILL_DRAW);
	bg->SetAlignment(B_ALIGN_LEFT);
	//ResizeBy(font.StringWidth(text)-50,height.ascent+height.descent-15);
	bg->SetViewColor(255,255,150,255);
	//bg->SetViewColor(B_TRANSPARENT_32_BIT);
	bg->MakeResizable(true);
	bg->MakeEditable(false);
	bg->MakeSelectable(false);
	AddChild(bg);
	BFont font;
	bg->GetFont(&font);
	font_height height;
	font.GetHeight(&height);
	ResizeTo(font.StringWidth(text)+6,height.ascent+height.descent+6);
	bg->SetText(text);
	// ---
	AddToSubset(owner->Window());
	Show();
} 
	
void KTipsWindow::Quit()
{
	//snooze(10000);
	BWindow::Quit();
}

// KTips <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

KTips::KTips(int32 ID,const char *tipsText,BView *owner,KTipsManager *manager)
:BMessageFilter(B_MOUSE_MOVED)
{
	fID=ID;
	fTipsText.SetTo(tipsText);
	fSlaved=false;
	fOwner=owner;
	fManager=manager;
	// ---
	//owner->AddFilter(this);
}

BView *KTips::Owner()
{
	return fOwner;
}

const char *KTips::Text()
{
	return fTipsText.String();
}

void KTips::SetText(const char *tipsText)
{
	fTipsText.SetTo(tipsText);
}
	
filter_result KTips::Filter(BMessage *message, BHandler **target)
{ 
	//message->PrintToStream();
	int32 transit=message->FindInt32("be:transit");
	BPoint where=message->FindPoint("be:view_where");
	int32 buttons=message->FindInt32("buttons");
	switch (transit) {
		case B_ENTERED_VIEW:{
			PRINT(("entered %s\n",(*target)->Name()));
			where=fOwner->ConvertToScreen(where);
			float bottom=fOwner->ConvertToScreen(fOwner->Bounds()).bottom;
			//if (bottom-where.y<50)
				where.y=bottom;
			if (buttons==0) fManager->ShowTips(this,where);
			break;}
		case B_EXITED_VIEW:
			PRINT(("exited %s\n",(*target)->Name()));
			if (true||!fOwner->Bounds().Contains(where)||fOwner->CountChildren()==0)
				fManager->HideTips();
			break;
	}
	return B_DISPATCH_MESSAGE;
}

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

#define K_TIPS_MAX_LATENCY 5

KTips *add_tips(const char *tips,BView *owner)
{
	tips_manager()->AddTips(tips,owner);
}

KTipsManager *tips_manager()
{
	KTipsManager::_init();
	return KTipsManager::fMainTipsManager;
}

void shutdown_tips_manager()
{
	if (KTipsManager::fMainTipsManager!=NULL&&KTipsManager::fMainTipsManager->Lock()) 
		KTipsManager::fMainTipsManager->Quit();
}

KTipsManager *KTipsManager::fMainTipsManager=NULL;

bool KTipsManager::_init()
{
	if (fMainTipsManager==NULL) {
		fMainTipsManager=new KTipsManager();
		return true;
	} else return false;
}

KTipsManager::KTipsManager():BLooper("tips manager")
{
	Run();
	//fPulser=NULL;
	fPulser=new BMessageRunner(BMessenger(this),new BMessage(B_PULSE),500000);
	// ---
	fEnabled=true;
	fTipsWindow=NULL;
	fLastID=fCurrentID=fID=0;
	fCurrentTips=NULL;
	fState=K_TIPS_IDLE;
	fWait=K_TIPS_MAX_LATENCY;
}

KTipsManager::~KTipsManager()
{
	//Quit();
}

void KTipsManager::Enable(bool flag)
{
	if (flag) {
		fEnabled=true;
	} else {
		HideTips(true);
		fEnabled=false;
	}
}

bool KTipsManager::IsEnabled()
{
	return fEnabled;
}

KTips *KTipsManager::AddTips(const char *tipsText,BView *owner)
{
	++fID;
	KTips *tips=create_tips(fID,tipsText,owner,owner);
	// ---
	BView *child; 
  	if ( child = owner->ChildAt(0) ) { 
		while ( child ) {
			create_tips(fID,tipsText,owner,child);
			child = child->NextSibling(); 
       	} 
   	}
   	return tips;
}

KTips *KTipsManager::create_tips(int32 ID,const char *tipsText,BView *owner,BView *target)
{
	KTips *tips=new KTips(ID,tipsText,owner,this);
	if (owner!=target) tips->fSlaved=true;
	target->AddFilter(tips);
	return tips;
}
	
void KTipsManager::ShowTips(KTips *tips,BPoint where)
{
	if (fEnabled&&Lock()) {
		if (tips->fID!=fCurrentID) {
			if (fCurrentID!=0) {
				HideTips(true);
			}
			fCurrentTips=tips;
			fWhere=where;
			fState=K_TIPS_SHOWING;
			Pulse();
		} else {
			fState=K_TIPS_IDLE;
		}
		Unlock();
	}
}
	
void KTipsManager::HideTips(bool force)
{
	if (Lock()) {
		if (force) {
			if (fTipsWindow!=NULL) {
				fTipsWindow->Lock();
				fTipsWindow->Quit();
			}
			fTipsWindow=NULL;
			fActive=false;
			fCurrentID=0;
			fCurrentTips=NULL;
			fState=K_TIPS_IDLE;
		} else {// delay
			fState=K_TIPS_HIDING;
		}
		Unlock();
	}
}
	
void KTipsManager::Pulse()
{
	if (Lock()) {
		if ((fState==K_TIPS_SHOWING)&&fCurrentTips!=NULL&&fWait<=0
			&&fCurrentTips->Owner()->Window()->IsFront()
			&&fLastID!=fCurrentTips->fID
			&&fCurrentTips->Owner()->LockLooper()) {
			// ---
			BPoint where;
			uint32 buttons;
			fCurrentTips->Owner()->GetMouse(&where,&buttons,false);
			if (buttons==0) {
				where=fCurrentTips->fOwner->ConvertToScreen(where);
				fTipsWindow=new KTipsWindow(fCurrentTips->fOwner,where,fCurrentTips->Text());
				fTipsWindow->AddToSubset(fCurrentTips->fOwner->Window());
				fActive=true;
				fLastID=fCurrentID=fCurrentTips->fID;
				fState=K_TIPS_IDLE;
			}
			fCurrentTips->Owner()->UnlockLooper();
		} else if (fState==K_TIPS_HIDING) {
			HideTips(true);
		} else if (fCurrentTips!=NULL&&fLastID==fCurrentTips->fID) {
			fLastID=0;
		} else if (fCurrentTips==NULL&&fWait<K_TIPS_MAX_LATENCY) {
			++fWait;++fWait;
		} else {
			--fWait;
		}
		Unlock();
	}
}

void KTipsManager::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case B_PULSE:
			this->Pulse();
			break;
		default:
			BLooper::MessageReceived(message);
	}
}
